#include <stdio.h>
#include <net/socket.h>


/* This function returns your current IP address.  It relies on your
   ppp-script to create and update the address file. */
unsigned long gethostaddr1(void)
{
   FILE *ipaddr_file;
   char ipaddr_str[16];

   /* If the IPADDRESS file does not exist then return the loopback
      address. */
   if (!(ipaddr_file = fopen("/boot/home/config/etc/IPADDRESS", "r"))) {
      return INADDR_LOOPBACK;
   }
	
   /* Read the current IP address from the file. */
   fscanf(ipaddr_file, "%s", ipaddr_str);
	
   fclose(ipaddr_file);
	
   return (inet_addr(ipaddr_str));
}


/* This function also returns your current IP address.  It relies on
   the ability to create a socket, bind that socket, then call the
   getsockname() function. */
unsigned long gethostaddr2(void)
{
   struct sockaddr_in addr;
   int sock, size;

   memset((char *)&addr, 0, sizeof(addr));
   addr.sin_family = AF_INET;
   addr.sin_addr.s_addr = INADDR_ANY;
   addr.sin_port = 0;

   if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
      printf("socket() failed in gethostaddr()\n");
      exit(0);
   }
	
   if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
      printf("bind() failed in gethostname()\n");
      closesocket(sock);
      exit(0);
   }

   /* This fills in the control_addr structure with the bound address
      information. */
   size = sizeof(addr);
   if ((getsockname(sock, (struct sockaddr *)&addr, &size)) < 0) {
      printf("getsockname() failed in gethostname()\n");
      closesocket(sock);
      exit(0);
   }

   closesocket(sock);

   if (addr.sin_addr.s_addr == INADDR_ANY)
      return INADDR_LOOPBACK;

   return addr.sin_addr.s_addr;
}


void main(void)
{
   struct in_addr addr;

   addr.s_addr = gethostaddr1();
   printf("gethostaddr1() : IP address = %s\n", inet_ntoa(addr));

   addr.s_addr = gethostaddr2();
   printf("gethostaddr2() : IP address = %s\n", inet_ntoa(addr));
}
